@@ -2,6 +2,8 @@
 #define HOOK_H
 #include "strvec.h"
 
+struct repository;
+
 struct run_hooks_opt
 {
 	/* Environment vars to be set for each hook */
@@ -30,6 +32,11 @@ struct run_hooks_opt
 	 * was invoked.
 	 */
 	int *invoked_hook;
+
+	/**
+	 * Path to file which should be piped to stdin for each hook.
+	 */
+	const char *path_to_stdin;
 };
 
 #define RUN_HOOKS_OPT_INIT { \
@@ -50,12 +57,12 @@ struct hook_cb_data {
  * or disabled. Note that this points to static storage that will be
  * overwritten by further calls to find_hook and run_hook_*.
  */
-const char *find_hook(const char *name);
+const char *find_hook(struct repository *r, const char *name);
 
 /**
  * A boolean version of find_hook()
  */
-int hook_exists(const char *hookname);
+int hook_exists(struct repository *r, const char *hookname);
 
 /**
  * Takes a `hook_name`, resolves it to a path with find_hook(), and
@@ -65,13 +72,14 @@ int hook_exists(const char *hookname);
  * Returns the status code of the run hook, or a negative value on
  * error().
  */
-int run_hooks_opt(const char *hook_name, struct run_hooks_opt *options);
+int run_hooks_opt(struct repository *r, const char *hook_name,
+		  struct run_hooks_opt *options);
 
 /**
  * A wrapper for run_hooks_opt() which provides a dummy "struct
  * run_hooks_opt" initialized with "RUN_HOOKS_OPT_INIT".
  */
-int run_hooks(const char *hook_name);
+int run_hooks(struct repository *r, const char *hook_name);
 
 /**
  * Like run_hooks(), a wrapper for run_hooks_opt().
@@ -81,5 +89,6 @@ int run_hooks(const char *hook_name);
  * argument. These things will be used as positional arguments to the
  * hook. This function behaves like the old run_hook_le() API.
  */
-int run_hooks_l(const char *hook_name, ...);
+LAST_ARG_MUST_BE_NULL
+int run_hooks_l(struct repository *r, const char *hook_name, ...);
 #endif
