@@ -2,6 +2,7 @@
 
 #include "git-compat-util.h"
 #include "config.h"
+#include "environment.h"
 #include "gettext.h"
 #include "hex.h"
 #include "http.h"
@@ -150,7 +151,7 @@ int cmd_main(int argc, const char **argv)
 
 	trace2_cmd_name("http-fetch");
 
-	git_config(git_default_config, NULL);
+	repo_config(the_repository, git_default_config, NULL);
 
 	if (packfile) {
 		if (!index_pack_args.nr)
