@@ -5,7 +5,6 @@ test_description='ls-files tests with relative paths
 This test runs git ls-files with various relative path arguments.
 '
 
-TEST_PASSES_SANITIZE_LEAK=true
 . ./test-lib.sh
 
 test_expect_success 'prepare' '
@@ -39,10 +38,7 @@ test_expect_success 'ls-files with mixed levels' '
 test_expect_success 'ls-files -c' '
 	(
 		cd top/sub &&
-		for f in ../y*
-		do
-			echo "error: pathspec $SQ$f$SQ did not match any file(s) known to git"
-		done >expect.err &&
+		printf "error: pathspec $SQ%s$SQ did not match any file(s) known to git\n" ../y* >expect.err &&
 		echo "Did you forget to ${SQ}git add${SQ}?" >>expect.err &&
 		ls ../x* >expect.out &&
 		test_must_fail git ls-files -c --error-unmatch ../[xy]* >actual.out 2>actual.err &&
@@ -54,10 +50,7 @@ test_expect_success 'ls-files -c' '
 test_expect_success 'ls-files -o' '
 	(
 		cd top/sub &&
-		for f in ../x*
-		do
-			echo "error: pathspec $SQ$f$SQ did not match any file(s) known to git"
-		done >expect.err &&
+		printf "error: pathspec $SQ%s$SQ did not match any file(s) known to git\n" ../x* >expect.err &&
 		echo "Did you forget to ${SQ}git add${SQ}?" >>expect.err &&
 		ls ../y* >expect.out &&
 		test_must_fail git ls-files -o --error-unmatch ../[xy]* >actual.out 2>actual.err &&
